/*
Adds XML and XPath related columns to Dopus
There are three fixed columns,

1: Is valid XML        : Is the file valid XML (true/false)
2: Root Node           : The name of the root node
3: Root Node Namespace : The Namespace of the root node.

The next two Columns are XPath columns. You should editing the XPath to suit your needs. You can create as many of these as you want.
*/

// -------------------- Options
var gbUseLogging=false;

// Define columns here, includes title and reg-exp.
var config = {
	columns: [{
		name:"IsValid",
		displayName:"Is valid XML2",
		type: "string",
		defwidth: 18
	}, {
		name:"RootNode",
		displayName:"Root Node",
		type: "string",
		defwidth: 18
	}, {
		name:"RootNodeNS",
		displayName:"Root Node Namespace",
		type: "string",
		defwidth: 18
	}]
};

//Default values for extendedConfig
var defaultCustomColumns = {
	columns: [{
		//Column settings
		// Columns setting details https://www.gpsoft.com.au/help/opus12/index.html#!Documents/Scripting/ScriptColumn.htm
		name:"Xpath1", 
		label:"Xpath1",
		header: "", 
		type: "string",
		defwidth: 20, //int or string.
		//grouporder:, //"Never Modified;Modified"
		justify: "left", //"left", "right", "center" or "path". The default is "left".
		//Not implemented yet
		infotiponly: false, //Not available for column
		//Not implemented yet
		maxstars: 5, // int
		datetimeformat: "yyyy-mm-dd", //Date Format used to parse the extracted date information (time not realy supported at the moment!)
		//Not implemented yet
		noGroup: false,
		grouporder: "",
		//enableAutoGroup: true,// true (default)
		//Not implemented yet
		sorting:"normal",
		//enableSort: false,//flag (true/false) as described in the DOpus documentation
		//defsort: true, //0 default, -1 reverse sort.
    
		//Add your xpath here
    	//E.G. Find the first node that is called nodename or anothername ignoring namespace
		xpath: ".//*[local-name() = 'nodename']"
	}, {
		name:"Xpath2", 
		label:"Xpath2",
		header: "Xpath2", 
		type: "string",
		defwidth: 20, //int or string.
		//grouporder:, //"Never Modified;Modified"
		justify: "left", //"left", "right", "center" or "path". The default is "left".
		//Not implemented yet
		infotiponly: false, //Not available for column
		//Not implemented yet
		maxstars: 5, // int
		datetimeformat: "yyyy-mm-dd", //Date Format used to parse the extracted date information (time not realy supported at the moment!)
		//Not implemented yet
		noGroup: false,
		grouporder: "",
		//enableAutoGroup: true,// true (default)
		//Not implemented yet
		sorting:"normal",
		//enableSort: false,//flag (true/false) as described in the DOpus documentation
		//defsort: true, //0 default, -1 reverse sort.
		//Add your xpath here
		xpath: "/",
	}]
};

// Functions
var doVars = DOpus.vars;
var doFsu = DOpus.FSUtil;
var doLogCmd = DOpus.NewCommand;
var configVarKey = "XmlCustomColumnsConfig";

function OnInit(initData){
	var uid = "6d4af44b-f50c-4029-a36d-0bfebbfb548d";
	var url = "http://resource.dopus.com/";
	initData.url = url;
	initData.name = "Xml & xpath columns";
	initData.desc = "Columns related to XML and XPath functionality.";
	initData.copyright = "wowbagger";
	initData.min_version = "12.0";
	initData.version = "0.4";
	initData.default_enable = true;

	// variable for enabling debug, so as to enable debug via a button
	initData.config_desc = DOpus.NewMap();

	initData.config.DEBUG = false;
	initData.config_desc("DEBUG") = "Set DEBUG flag to true in order to enable logging messages to the Opus Output Window";

	initData.config.DefaultFileFilter = ".*\\.xml";
	initData.config_desc("DefaultFileFilter") = "Sets teh default file filter";
}

function OnAddCommands(addCmdData)
{
	var cmd = addCmdData.AddCommand();
	cmd.name = "XmlColumnsConfigure";
	cmd.method = "OnXmlColumnsConfigure";
	cmd.desc = "Configure the Xml Custom columns";
	cmd.label = "XmlColumnsConfigure";
}

// The OnAddColumns event is called to allow your script add-in to add columns. 
// Call the AddColData.AddColumn method once for each column you wish to add.
function OnAddColumns(addColData)
{
	//LogMessage("OnAddColumns called", true)

	//Add fixed columns
	for(i=0;i<config.columns.length;i++) {
		AddColumn(addColData, //iAddColumn
			config.columns[i].name, //name
			config.columns[i].label, //label
			config.columns[i].header, //label
			config.columns[i].type, //type
			config.columns[i].defwidth, //defwidth
			config.columns[i].grouporder,
			config.columns[i].sorting,
			config.columns[i].autorefresh, //autorefresh
			false,
			config.columns[i].maxStars,
			config.columns[i].justify, //justify
			config.columns[i].infotiponly,
			true //multicol
      );
	}

	//Add custom columns
	var customColumns = LoadCustomColumns();
	if(customColumns && customColumns.columns.length > 0)
	{
		//var customColumns = JSON.parse(Script.config.CustomColumns);
		for(i=0;i<customColumns.columns.length;i++) {
			var currentCol = customColumns.columns[i];

			if(addColData,currentCol)
			{
				AddColumn(addColData, //iAddColumn
					currentCol.name, //name
					currentCol.label, //label
					currentCol.header, //label
					currentCol.type, //type
					currentCol.defwidth, //defwidth
					currentCol.grouporder,
					currentCol.sorting,
					currentCol.autorefresh, //autorefresh
					false,
					currentCol.maxStars,
					currentCol.justify, //justify
					currentCol.infotiponly,
					true //multicol
				);
			}
		}
	}
}

function AddColumn(iAddColumn, name, label, header, type, defwidth, grouporder, sorting, autorefresh, nameRefresh, maxStars, justify, infotiponly, multicol){
	var autoGroup = ((typeof grouporder === 'undefined') || grouporder  == "");

	LogMessage("AddColumn: name=" + name + " label=" + label + " header=" + header + 
				" type=" + type + " defwidth=" + defwidth + 
				" grouporder=" + grouporder + " autoGroup=" + autoGroup + " sorting=" + sorting + 
				" autorefresh=" + autorefresh + " maxStars=" + maxStars + " justify=" + justify + 
				" multicol=" + multicol + " infotiponly=" + infotiponly);

	//https://www.gpsoft.com.au/help/opus12/index.html#!Documents/Scripting/ScriptColumn.htm

	var col;
	col=iAddColumn.AddColumn();
	col.name = name;
	col.label = label;
	col.header = header;

	col.type = type;
	col.method = "OnXml";
	col.namerefresh = nameRefresh;
	col.autogroup = autoGroup;///true;
	col.autorefresh = autorefresh;///true;
	col.justify = justify;///"left";
	col.multicol = multicol;///true;
	col.infotiponly = infotiponly;
	col.maxstars = maxStars;
	col.grouporder = grouporder;
	col.nogroup = ((typeof grouporder === 'undefined') || grouporder  == "");
	col.defsort = (sorting == "reverse") ? -1 : 1;
	col.nosort = (sorting == "disabled");
	
	/*
	LogMessage("AddColumn: name=" + name + " label=" + label + " type=" + type + " defwidth=" + defwidth)
	var col;
	col=iAddColumn.AddColumn();
	col.name = name;
	col.label = label;
	col.type = type;
	col.method = "OnCustomText";
	col.autogroup = true;
	col.autorefresh = true;
	col.justify = "left";
	col.multicol = true;*/
	if(defwidth != null && defwidth != 0)
	{
		col.defwidth=defwidth;
	}  
}

// The OnScriptConfigChange event can be implemented by a script add-in to receive 
// notification whenever the user modifies the script's configuration via the Preferences editor.
function OnScriptConfigChange(configChangeData)
{
	LogMessage("OnScriptConfigChange " + (configChangeData ? (configChangeData.changed ? configChangeData.changed.length : 'nothing changed') :'null' , true));
	for (var i = 0; i < configChangeData.changed.length; i++) {
		LogMessage("OnScriptConfigChange " + i + ":" + configChangeData.changed[i], true)
	}

	Script.InitColumns;
}


// Implement the ExternalCompare command
function OnXmlColumnsConfigure(scriptCmdData)
{
    var dlg = DOpus.Dlg;
    
    dlg.window = scriptCmdData.func.sourcetab;
    dlg.template = "XmlColumnsConfigure";
    dlg.detach = true;
    dlg.Show();
    onDialogPopulateColumnsList(dlg);

    var closeDialog;
    while (!closeDialog) {
  		var msg = dlg.GetMsg();
  		if (!msg.result) break;
  		//LogMessage("Msg: " + msg.control + " value: " + msg.value + " event:" + msg.event);
  		switch (msg.control) {
  			case "columnsList" :
  				onDialogConfigSelected(dlg, msg);
  				break;
  			case "buttonAdd" :
  				onDialogAddColumn(dlg, msg);
  				break;
  			case "buttonDelete" :
  				onDialogDeleteColumn(dlg, msg);
  				break;
  			case "buttonApply" :
  				 onDialogApplyChanges(dlg, msg);
  				break;
  			case "buttonClipboard" :
  				 CopyToClipboard();
  				break;
  			case "columnType" :
  				onDialogColumnType(dlg);
  				break;
  			case "columnInput" :
  				onDialogColumnInput(dlg);
  				break;
  			case "columnFilter" :
  				onDialogColumnFilter(dlg);
  				break;
  			case "columnSetNoGroup" :
  				onSetNoGroup(dlg);
  				break;
  			case "buttonClearSettings" :
  				ClearCustomColumns();
  				closeDialog = true;
  				break;

  			case "buttonOK" :
  				//params = DialogClosed(dlg, msg);
  				closeDialog = true;
  				break;
  		} 
    }
}

function onDialogPopulateColumnsList(dlg)
{
    var columnsList = dlg.Control("columnsList");
    var customColumnsConfig = LoadCustomColumns();//JSON.parse(Script.config.CustomColumns);

	var selectedColumn;
    for(var i = 0; i < customColumnsConfig.columns.length; ++i) 
    {
    	if(customColumnsConfig.columns[i])
    	{
			var location = columnsList.AddItem(customColumnsConfig.columns[i].name);
			var item = columnsList.GetItemAt(location);
			if(!selectedColumn)
			{ 
				item.selected = true;
				selectedColumn = customColumnsConfig.columns[i];
				onDialogConfigSelected(dlg);
			}
 			LogMessage("item: " + item.index + " location: " + location + " product:" + customColumnsConfig.columns[i].name);
 		}
    }
}

// Dialog functions
/*
function onDialogColumnFilter(dlg)
{
	var selectedType = GetSelectedItem(dlg ,"columnFilter");
	LogMessage("onDialogColumnFilter: " + ((selectedType && selectedType.name) ? selectedType.name : 'null'));

	var columnType = (typeof(selectedType) != "undefined") ? selectedType.name : "" ;

	dlg.Control("columnCustomFilterStatic").enabled = (columnType == "custom");
	dlg.Control("columnCustomFilter").enabled = (columnType == "custom");
}*/		

function onDialogColumnInput(dlg)
{
	var selectedType = GetSelectedItem(dlg ,"columnInput");
	LogMessage("onDialogColumnInput: " + ((selectedType && selectedType.name) ? selectedType.name : 'null'));

	var columnType = (typeof(selectedType) != "undefined") ? selectedType.name : "file" ;
	
	dlg.Control("columnFileFilterStatic").enabled = (columnType == "file");
	dlg.Control("columnFileFilter").enabled = (columnType == "file");
	dlg.Control("columnFileFilterStatic").visible = (columnType == "file");
	dlg.Control("columnFileFilter").visible = (columnType == "file");

	dlg.Control("columnFolderFilterStatic").enabled = (columnType == "folder");
	dlg.Control("columnFolderFilter").enabled = (columnType == "folder");
	dlg.Control("columnFolderFilterStatic").visible = (columnType == "folder");
	dlg.Control("columnFolderFilter").visible = (columnType == "folder");
}

function onDialogColumnType(dlg)
{
	var selectedType = GetSelectedItem(dlg ,"columnType");
	LogMessage("onDialogColumnType: " + ((selectedType && selectedType.name) ? selectedType.name : 'null'));

	var columnType = (typeof(selectedType) != "undefined") ?  selectedType.name : "" ;

	dlg.Control("columnDateTimeFormat").enabled = (columnType == "datetime" || columnType == "date" || columnType == "time" );
	dlg.Control("columnDateTimeFormatStatic").enabled = (columnType == "datetime" || columnType == "date" || columnType == "time" );
	dlg.Control("columnDateTimeFormat").visible = (columnType != "stars");
	dlg.Control("columnDateTimeFormatStatic").visible = (columnType != "stars");

	dlg.Control("columnMaxStarsStatic").enabled = (columnType == "stars");
	dlg.Control("columnMaxStars").enabled = (columnType == "stars");
	dlg.Control("columnMaxStarsStatic").visible = (columnType == "stars");
	dlg.Control("columnMaxStars").visible = (columnType == "stars");
}

function onSetNoGroup(dlg)
{
	var checkBox = dlg.Control("columnSetNoGroup");
	LogMessage("onSetNoGroup: " + checkBox.name  + "  " + checkBox.value);
	dlg.Control("columnGroupOrder").enabled = !(checkBox.value);
}

function onDialogConfigSelected(dlg)
{
	var selectedUiColumn = GetSelectedItem(dlg ,"columnsList");
	LogMessage("onDialogConfigSelected: " + ((selectedUiColumn && selectedUiColumn.name) ? selectedUiColumn.name : 'null'));
	if(typeof(selectedUiColumn) == "undefined") return;

	var selectedColumnConfig = GetXmlColumnConfig(selectedUiColumn.name);
  
	dlg.Control("columnName").value = selectedColumnConfig ? selectedColumnConfig.name : "";
	dlg.Control("columnLabel").value = selectedColumnConfig ? selectedColumnConfig.label : "";
	dlg.Control("columnHeader").value = selectedColumnConfig ? selectedColumnConfig.header : "";
  	SetSelectedItem(dlg, "columnType", (selectedColumnConfig ? selectedColumnConfig.type : ""));
	onDialogColumnType(dlg);
	dlg.Control("columnDateTimeFormat").value = selectedColumnConfig ? selectedColumnConfig.datetimeformat : "yyyy-mm-dd";
	dlg.Control("columnMaxStars").value = selectedColumnConfig ? selectedColumnConfig.maxstars : 5;
	SetSelectedItem(dlg, "columnJustify", (selectedColumnConfig ? selectedColumnConfig.justify : ""));
	dlg.Control("columnDefaultWidth").value = selectedColumnConfig ? selectedColumnConfig.defwidth : "";
	dlg.Control("columnSetNoGroup").value = selectedColumnConfig ? selectedColumnConfig.nogroup : false;
	dlg.Control("columnGroupOrder").value = selectedColumnConfig ? selectedColumnConfig.grouporder : "";
	SetSelectedItem(dlg, "columnSorting", (selectedColumnConfig ? selectedColumnConfig.sorting : "normal"));
	dlg.Control("columnSetInfoTipOnly").value = selectedColumnConfig ? selectedColumnConfig.infotiponly : false;
 
	dlg.Control("columnXpath").value = selectedColumnConfig ? selectedColumnConfig.xpath : "";
	var colInputValue = selectedColumnConfig ? selectedColumnConfig.input : "file";
	SetSelectedItem(dlg, "columnInput", colInputValue);
	onDialogColumnInput(dlg);
	dlg.Control("columnFileFilter").value = selectedColumnConfig.fileFilter;
	dlg.Control("columnFolderFilter").value = selectedColumnConfig.folderFile;
	/*
	var colFilterValue = selectedColumnConfig ? selectedColumnConfig.filter : "";
	if(colFilterValue == "all" || colFilterValue == "files" || colFilterValue == "folders")
	{
		SetSelectedItem(dlg, "columnFilter", colFilterValue);
		dlg.Control("columnCustomFilter").value = "";
	}
	else
	{
		SetSelectedItem(dlg, "columnFilter", "custom");
		dlg.Control("columnCustomFilter").value = colFilterValue;
	}
	onDialogColumnFilter(dlg);
	*/
}

function onDialogApplyChanges(dlg, msg)
{
	var selectedUiColumn = GetSelectedItem(dlg ,"columnsList");
	//LogMessage("onDialogApplyChanges: " + selectedUiColumn ? selectedUiColumn.name : 'null');
	if(!selectedUiColumn) return;
	
	var oldName = selectedUiColumn.name;
  	
	var customColumns = LoadCustomColumns();
	for(var i = 0; i < customColumns.columns.length; ++i) {
		if(customColumns.columns[i] && customColumns.columns[i].name.toLowerCase() == oldName.toLowerCase()) {
			var selectedColumn = customColumns.columns[i];
			selectedColumn.name = dlg.Control("columnName").value;
			selectedColumn.label = dlg.Control("columnLabel").value;
			selectedColumn.header = dlg.Control("columnHeader").value;
			var selectedType = GetSelectedItem(dlg, "columnType");
			selectedColumn.type = selectedType ? selectedType.name : "";
			selectedColumn.datetimeformat = dlg.Control("columnDateTimeFormat").value;
			selectedColumn.maxstars = dlg.Control("columnMaxStars").value;
			var selectedJustify = GetSelectedItem(dlg, "columnJustify");
			selectedColumn.justify = selectedJustify ? selectedJustify.name : "";
			selectedColumn.defwidth = dlg.Control("columnDefaultWidth").value;
			selectedColumn.nogroup = dlg.Control("columnSetNoGroup").value;
			selectedColumn.grouporder = dlg.Control("columnGroupOrder").value;
			var selectedSorting = GetSelectedItem(dlg, "columnSorting");
			selectedColumn.sorting = selectedSorting ? selectedSorting.name : "";
			selectedColumn.infotiponly = dlg.Control("columnSetInfoTipOnly").value;
		  	selectedColumn.xpath = dlg.Control("columnXpath").value;
	    	var selectedInput = GetSelectedItem(dlg, "columnInput");
			selectedColumn.input = selectedInput ? selectedInput.name : "file";
	    	selectedColumn.fileFilter = dlg.Control("columnFileFilter").value;
			selectedColumn.folderFile = dlg.Control("columnFolderFilter").value;

	    	/*
			var selectedFilter = GetSelectedItem(dlg, "columnFilter");
			var SelectedFilterValue = selectedFilter ? selectedFilter.name : "";
			if(SelectedFilterValue == "custom")
			{
				selectedColumn.filter = dlg.Control("columnCustomFilter").value;
			}
			else
			{
				selectedColumn.filter = SelectedFilterValue;
			}
			*/
			SaveCustomColumns(customColumns);
	
			//Script.RefreshColumn(oldName.name);
			selectedUiColumn.name = dlg.Control("columnName").value; 
			Script.RefreshColumn(dlg.Control("columnName").value);

		  	break;
		}
	}
}

function onDialogAddColumn(dlg, msg)
{
  	//LogMessage("onDialogAddColumn");
  
  	var newColumn = defaultCustomColumns.columns[0];
  	var customColumnsConfig = LoadCustomColumns();
	customColumnsConfig.columns.push(newColumn);
	SaveCustomColumns(customColumnsConfig);

	//customColumnsConfig.columns[customColumnsConfig.columns.length - 1]
	AddToColumnsList(dlg, newColumn.name, true);
	onDialogConfigSelected(dlg, msg);
}

function onDialogDeleteColumn(dlg, msg)
{
	var selectedUiColumn = GetSelectedItem(dlg ,"columnsList");
	//LogMessage("onDialogApplyChanges: " + selectedUiColumn ? selectedUiColumn.name : 'null');
	if(!selectedUiColumn) return;

	var customColumns = LoadCustomColumns();
	for(var i = 0; i < customColumns.columns.length; ++i) {
		if(customColumns.columns[i].name.toLowerCase() == selectedUiColumn.name.toLowerCase()) {
			
			delete customColumns.columns[i];

			SaveCustomColumns(customColumns);
			Script.RefreshColumn(selectedUiColumn.name);
		  	break;
		}
	}

	RemoveFromColumnsList(dlg, selectedUiColumn);
	onDialogConfigSelected(dlg, msg);
}

function RemoveFromColumnsList(dlg, column)
{
	var columnsList = dlg.Control("columnsList");
	var index = column.index;
	columnsList.RemoveItem(column);

	if(columnsList.count > 0)
	{
		var selectedColumn = columnsList.GetItemAt((index > 0) ? index -1 : 0);
    	selectedColumn.selected = true;
	}
}

function AddToColumnsList(dlg, name, setSelected)
{
	var columnsList = dlg.Control("columnsList");
 	var location = columnsList.AddItem(name);
 	if(setSelected)
 	{
		var item = columnsList.GetItemAt(location);
		item.selected = true;
	}
}

function GetSelectedItem(dlg, controlName)
{
	LogMessage("GetSelectedItem: " + controlName);

	var listItems = dlg.Control(controlName);
  for(var i = 0; i < listItems.count; ++i) {
  	var column = listItems.GetItemAt(i);
	
	  //LogMessage("GetSelectedItem: " + controlName + " " + i + " of " + listItems.count + " is" + (column.selected ? "selected" : "not selected"));
  	if(column && column.selected)
  	{
		  //LogMessage("GetSelectedItem: found " + column.name);
  		return column;
	  }
  }
}

function SetSelectedItem(dlg, controlName, value)
{
	var listItems = dlg.Control(controlName);

	if(typeof(value) == "undefined") 
	{
		var item = listItems.GetItemAt(0);
		//LogMessage("SetSelectedItem: " + controlName + ":defaulted to " + item.name);
		item.selected = true;
		return;
	}

    for(var i = 0; i < listItems.count; ++i) {
    	var item = listItems.GetItemAt(i);
	
		//LogMessage("SetSelectedItem: " + controlName + " " + i + " of " + listItems.count + " is " + item.name  + " looking for " + value);
    	if(item && item.name == value)
    	{
			//LogMessage("SetSelectedItem: " + controlName + ":found " + item.name);
			item.selected = true;
 		}
    }
}

function CopyToClipboard(){

	var customColumns = LoadCustomColumns();
	var config = JSON.stringify(customColumns, undefined, 2).replace(/(\n)+/g,"\r\n");
	var doCmd = DOpus.NewCommand;

	var cmd = "Clipboard SET " + config;
	doCmd.RunCommand(cmd);
}

function SaveCustomColumns(customColumns){

	var newList = new Array();
	for(var i = 0; i < customColumns.columns.length; ++i) {
		if(customColumns.columns[i]) {
			newList.push(customColumns.columns[i]);
		}
	}

	customColumns.columns = newList;

	var config = JSON.stringify(customColumns, undefined, 2).replace(/(\n)+/g,"\r\n");

	LogMessage("SaveCustomColumns: " + config);
	doVars(configVarKey) = config;
    doVars(configVarKey).persist = true;

    Script.InitColumns;
}

function ClearCustomColumns(){
	if(doVars.Exists(configVarKey))
	{
        doVars(configVarKey).Delete()
	}
}

function LoadCustomColumns(){
	//LogMessage("LoadCustomColumnsConfiguration from var");
	//return JSON.parse(Script.config.CustomColumns);
  	var config = doVars.Exists(configVarKey) ? JSON.parse(doVars.Get(configVarKey)) : null;
  	if(!config) 
  	{
  		config = defaultCustomColumns;
  	}

	return config
}

function GetXmlColumnConfig(name)
{
	var customColumns = LoadCustomColumns();
	for(var i = 0; i < customColumns.columns.length; ++i) {
		if(customColumns.columns[i].name.toLowerCase() == name.toLowerCase()) {
			return customColumns.columns[i];
		}
	}
}

function IsDebugEnabled()
{
  
  if(Script)
  {
	return gbUseLogging || Script.config.DEBUG || doLogCmd.IsSet(Script.config.DebugEnableVar);
  }

  return gbUseLogging;
}

function LogMessage(message, force)
{
  if (force || IsDebugEnabled()) { DOpus.OutputString(message)};
}

function OnXml(scriptColData){
	var itemPath = scriptColData.item.path + "\\" + scriptColData.item.name;
	LogMessage("OnXml: column=" + scriptColData.col + " columnsCount=" + scriptColData.columns.length + " itempPath=" + itemPath);
	if (itemPath.length > 255) return;

	var columnsConfig = new Array();
	for (var i = 0; i < config.columns.length; ++i) {
		if(config.columns[i]) {
			columnsConfig.push(config.columns[i]);
		}
	}
	
	var customColumnsConfig = LoadCustomColumns();
	for(var i = 0; i < customColumnsConfig.columns.length; ++i) {
		if(customColumnsConfig.columns[i]) {
			columnsConfig.push(customColumnsConfig.columns[i]);
		}
	}
	
	if (!InArray(scriptColData.col,columnsConfig)) return;
	
	LogMessage("Scanning xml columns:" + columnsConfig.length + " file:" + itemPath)
	if (scriptColData.item.is_dir)
	{
		var results = ScanFolderForXml(itemPath, columnsConfig);
	}
	else 
	{
		if((new RegExp(Script.config.DefaultFileFilter)).test(itemPath))
		{
			var results = ScanXmlFile(itemPath, columnsConfig);
		}
	}

  	if (results){
  		for ( i = 0; i < columnsConfig.length; i++) 
  		{
  			if (typeof(columnsConfig[i].name) != "undefined")
  			{
  				if (scriptColData.columns.exists(columnsConfig[i].name))
  				{
  					var result = results[columnsConfig[i].name];

  					switch (columnsConfig[i].type) {
  						case "date" :
  						case "time" :
  						case "datetime" :
  							LogMessage("Datetime for " + columnsConfig[i].name + "[" + columnsConfig[i].type + "]:" + result);
  							var parsedDate = parseDate(results[columnsConfig[i].name], columnsConfig[i].datetimeformat);
  							scriptColData.columns(columnsConfig[i].name).value = parsedDate ? parsedDate.getVarDate() : null;
  							break;
  						case "graph" :
  						case "igraph" :
  						case "percent" :
  						case "stars" :
  							LogMessage("number for " + columnsConfig[i].name + "[" + columnsConfig[i].type +"]:" + result);
  	    					scriptColData.columns(columnsConfig[i].name).value = (isNaN(result) ? 0 : result);
  							break;
  						default :
  							LogMessage("results for " + columnsConfig[i].name + "[" + columnsConfig[i].type +"]:" + result);
      						scriptColData.columns(columnsConfig[i].name).value = result;
  							break;
  					} 
  				}
  				else
  				{
  					LogMessage("column not loaded " +columnsConfig[i].name + "["+columnsConfig[i].type +"]:" + result);
  				}
  			}
  		}
  	}
}

function parseDate(input, format) {
	if(!input) return;

	var parsedDate = new Date(Date.parse(input));
	if(!parsedDate || parsedDate == "NaN")
    {

		format = format || 'yyyy-mm-dd'; // default format
		var parts = input.match(/(\d+)/g), 
		  i = 0, fmt = {};
	  	if(parts)
		{
			// extract date-part indexes from the format
			format.replace(/(yyyy|dd|mm)/g, function(part) { fmt[part] = i++; });
			parsedDate = new Date(parts[fmt['yyyy']], parts[fmt['mm']]-1, parts[fmt['dd']]);	
		}
    }
				
	return (!parsedDate || parsedDate == "NaN") ? null : parsedDate;
}

function ScanFolderForXml(folderPath, columnsConfig)
{
	var result = {};
	var msxml = new ActiveXObject("MSXML2.DOMDocument.6.0");
	
	LogMessage("ScanFolderForXml:" + columnsConfig.length + " file:" + folderPath)
	
	for(i=3;i<columnsConfig.length;i++)
	{	
		if(columnsConfig[i].input == "folder")
		{
			var filePath = folderPath + "\\"  +  columnsConfig[i].folderFile;

			if(doFsu.Exists(filePath))
			{
				msxml.load(filePath);

				if (msxml.parseError.errorCode != 0)
				{
					LogMessage("XML load error code: " + msxml.parseError.errorCode)
				}
				else 
				{
					LogMessage("looking for xpath["+i+"]: " + columnsConfig[i].xpath)
					if(columnsConfig[i].xpath)
					{
					  	LogMessage("looking for xpath: " + columnsConfig[i].xpath)
						var node = msxml.documentElement.selectSingleNode(columnsConfig[i].xpath);
						if(node)
						{
							result[columnsConfig[i].name] = node.text;
						}
					}
				}
			}
		}
  	}
	
	if (result)
		return result;
	else
		return null;
}

function ScanXmlFile(filePath, columnsConfig)
{
	//if (scriptColData.item.ext.toLowerCase() == Script.config.DefaultFileFilter)
	//  	selectedColumn.fileFilter = dlg.Control("columnFileFilter").value;


	var result = {};
	var msxml = new ActiveXObject("MSXML2.DOMDocument.6.0");
	
	LogMessage("ScanXmlFile:" + columnsConfig.length + " file:" + filePath)
			
	//Load and validate the specified file into the DOM.
	msxml.load(filePath);
	
	if (msxml.parseError.errorCode != 0)
	{
    	LogMessage("XML load error code: " + msxml.parseError.errorCode)
		result[columnsConfig[0].name] = "false";
	}
	else 
	{
    	result[columnsConfig[0].name] = "true";
		result[columnsConfig[1].name] = msxml.documentElement.nodeName;
		result[columnsConfig[2].name] = msxml.documentElement.namespaceURI;
		
		for(i=3;i<columnsConfig.length;i++)
		{
			var filter = (columnsConfig[i].fileFilter == "" ? Script.config.DefaultFileFilter : columnsConfig[i].fileFilter);
			
			if(columnsConfig[i].input != "folder" && (new RegExp(filter)).test(filePath))
			{
				LogMessage("looking for xpath["+i+"]: " + columnsConfig[i].xpath)
				if(columnsConfig[i].xpath)
				{
				  	LogMessage("looking for xpath: " + columnsConfig[i].xpath)
					var node = msxml.documentElement.selectSingleNode(columnsConfig[i].xpath);
					if(node)
					{
						result[columnsConfig[i].name] = node.text;
					}
				}
			}
		}
	}

	if (result)
		return result;
	else
		return null;
}

function InArray(col,arr){
	for (var i=0;i<arr.length;i++){
	if (col==arr[i].name) return true;
	}
}

==SCRIPT RESOURCES
<resources>
	<resource name="XmlColumnsConfigure" type="dialog">
		<dialog fontsize="8" height="330" lang="english" resize="yes" title="XML and Xpath custom columns" width="318">
			<control close="0" height="14" name="buttonClose" resize="xy" title="Close" type="button" width="50" x="252" y="306" />
			<control height="318" name="columnsGroup" resize="h" title="Xpath Columns" type="group" width="96" x="6" y="6" />
			<control height="252" name="columnsList" resize="h" type="listbox" width="84" x="12" y="18" />
			<control header="yes" height="64" name="groupColumnSettings" resize="w" title="Column Settings" type="group" width="204" x="108" y="12" />
			<control halign="left" height="8" name="columnNameSatic" title="Name" type="static" width="66" x="108" y="24" />
			<control halign="left" height="12" name="columnName" resize="w" tip="Not set" type="edit" width="132" x="174" y="24" />
			<control halign="left" height="8" name="columnLabelStatic" title="Label" type="static" width="66" x="108" y="42" />
			<control halign="left" height="12" name="columnLabel" resize="w" tip="Not set (defaults to Name)" type="edit" width="132" x="174" y="42" />
			<control halign="left" height="8" name="columnHeaderStatic" title="Header" type="static" width="66" x="108" y="60" />
			<control halign="left" height="12" name="columnHeader" resize="w" tip="Not set (defaults to Display Name)" type="edit" width="132" x="174" y="60" />
			<control halign="left" height="8" name="columnTypeStatic" title="Column Type" type="static" width="66" x="108" y="78" />
			<control height="40" name="columnType" type="combo" width="102" x="174" y="78">
				<contents>
					<item text="text" />
					<item text="number" />
					<item text="double" />
					<item text="size" />
					<item text="zip" />
					<item text="percent" />
					<item text="graph" />
					<item text="igraph" />
					<item text="date" />
					<item text="time" />
					<item text="datetime" />
					<item text="stars" />
				</contents>
			</control>
			<control halign="left" height="8" name="columnDateTimeFormatStatic" title="Date Time Format" type="static" width="60" x="108" y="96" />
			<control enable="no" halign="left" height="12" name="columnDateTimeFormat" tip="yyyy-MM-dd" type="edit" width="102" x="174" y="96" />
			<control halign="left" height="8" name="columnMaxStarsStatic" title="Max Stars" type="static" visible="no" width="60" x="108" y="96" />
			<control enable="no" halign="left" height="12" name="columnMaxStars" number="yes" tip="5" type="edit" visible="no" width="102" x="174" y="96" />
			<control halign="left" height="8" name="columnJustifyStatic" title="Column Justify" type="static" width="66" x="108" y="114" />
			<control height="40" name="columnJustify" type="combo" width="102" x="174" y="114">
				<contents>
					<item text="left" />
					<item text="right" />
					<item text="center " />
					<item text="path" />
				</contents>
			</control>
			<control halign="left" height="8" name="columnDefaultWidthStatic" title="Default Width" type="static" width="66" x="108" y="132" />
			<control halign="left" height="12" name="columnDefaultWidth" number="yes" tip="Not set" title="80" type="edit" width="66" x="174" y="132" />
			<control height="10" name="columnSetNoGroup" title="Disable Grouping" type="check" width="78" x="174" y="150" />
			<control halign="left" height="8" name="columnGroupOrderStatic" title="Group Order" type="static" width="66" x="108" y="162" />
			<control halign="left" height="12" name="columnGroupOrder" resize="w" tip="e.g. &quot;Never Modified;Modified&quot;" type="edit" width="132" x="174" y="162" />
			<control halign="left" height="8" name="columnSortingStatic" title="Sorting" type="static" width="66" x="108" y="180" />
			<control height="40" name="columnSorting" type="combo" width="102" x="174" y="180">
				<contents>
					<item text="normal" />
					<item text="reverse" />
					<item text="disabled" />
				</contents>
			</control>
			<control height="10" name="columnSetInfoTipOnly" title="Info Tip only" type="check" width="84" x="174" y="198" />
			<control header="yes" height="64" name="groupregXpath" resize="w" title="Xpath  Settings" type="group" width="204" x="108" y="210" />
			<control halign="left" height="8" name="columnXpathStatic" title="XPath" type="static" width="66" x="108" y="222" />
			<control halign="left" height="12" name="columnXpath" resize="w" tip="Not set I.E. ./" type="edit" width="132" x="174" y="222" />
			<control halign="left" height="8" name="columnInputStatic" title="Input" type="static" width="66" x="108" y="240" />
			<control height="40" name="columnInput" type="combo" width="102" x="174" y="240">
				<contents>
					<item text="file" />
					<item text="folder" />
				</contents>
			</control>
			<control header="yes" height="64" name="groupButtons" resize="yw" type="group" width="210" x="102" y="294" />
			<control default="yes" height="14" name="buttonApply" resize="xy" title="Appy Changes" type="button" width="74" x="234" y="276" />
			<control height="14" name="buttonAdd" resize="y" title="Add Column" type="button" width="60" x="24" y="282" />
			<control height="14" name="buttonDelete" resize="y" title="Delete Column" type="button" width="60" x="24" y="300" />
			<control height="14" name="buttonClipboard" resize="y" title="Copy settings to clipboard" type="button" width="102" x="114" y="306" />
			<control enable="no" halign="left" height="8" name="columnFolderFilterStatic" title="File Name" type="static" width="66" x="108" y="258" />
			<control enable="no" halign="left" height="12" name="columnFolderFilter" resize="w" tip="I.E. folder.xml" type="edit" width="132" x="174" y="258" />
			<control enable="no" halign="left" height="12" name="columnFileFilter" resize="w" tip="I.E. .*\.xml" type="edit" width="132" x="174" y="258" />
			<control enable="no" halign="left" height="8" name="columnFileFilterStatic" title="File Filter" type="static" width="66" x="108" y="258" />
		</dialog>
	</resource>
</resources>

